﻿using Advania.Signet.Cloud2Connector.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Advania.Signet.Cloud2Connector
{
    public class Connector
    {
        private string _username;
        private string _password;
        private string _url;
        private X509Certificate2 _cert;

        /// <summary>
        /// Cloud helper constructor
        /// </summary>
        /// <param name="url">Webservice REST url</param>        
        /// <param name="certificate">Signet certificate</param>
        public Connector(string url, string certificate, string username, string password)
        {
            _url = url;
            _cert = CommonHelper.GetCertificate(certificate);
            _username = username;
            _password = password;
        }

        /// <summary>
        /// Ping Signet Cloud API
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public string Ping(string message)
        {
            PingRequest request = new PingRequest { input = message, username = _username, password = _password };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "Ping", _cert);
            return JsonConvert.DeserializeObject<string>(stringResponse);            
        }

        /// <summary>
        /// Add document(s) to Signet
        /// </summary>
        /// <param name="documents"></param>
        /// <param name="notes"></param>
        /// <param name="metadata"></param>
        /// <param name="reason"></param>
        /// <param name="signers"></param>
        /// <param name="ordered"></param>
        /// <param name="startSigning"></param>
        /// <param name="endSigning"></param>
        /// <returns></returns>
        public AddDocumentResponse AddDocuments(List<Document> documents, string notes, string metadata, string reason, List<DocumentSigner> signers, bool ordered, bool dontadd, string groupId, DateTime? startSigning, DateTime? endSigning)
        {
            AddDocumentRequest req = new AddDocumentRequest
            {
                DocumentNotes = notes,
                Signers = signers,
                Documents = documents,
                Metadata = metadata,
                Reason = reason,
                StartSigning = startSigning,
                EndSigning = endSigning,
                Ordered = ordered,
                MininumSigners = 0,
                DontAddPage = dontadd
            };
            AddRequest request = new AddRequest { GroupID = groupId, request = req, username = _username, password = _password };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "AddDocument", _cert, true);
            return JsonConvert.DeserializeObject<AddDocumentResponse>(stringResponse);
        }

        /// <summary>
        /// Download document from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public List<Document> DownloadDocument(string docId, string groupId)
        {
            DocumentRequest request = new DocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "DownloadDocument", _cert);
            return JsonConvert.DeserializeObject<List<Document>>(stringResponse);
        }

        /// <summary>
        /// Get document info from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public List<DocumentInfo> GetDocumentInfo(string docId, string groupId)
        {
            DocumentRequest request = new DocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetDocument", _cert);
            return JsonConvert.DeserializeObject<List<DocumentInfo>>(stringResponse);
        }

        /// <summary>
        /// Get documents from Signet
        /// </summary>
        /// <param name="ssn"></param>
        /// <param name="fromTime"></param>
        /// <param name="toTime"></param>
        /// <param name="status"></param>
        /// <returns></returns>
        public List<DocumentInfo> GetDocuments(string ssn, DateTime? fromTime, DateTime? toTime, DocumentStatus? status)
        {
            DocumentsRequest request = new DocumentsRequest {SSN = ssn, Fromtime = fromTime, ToTime = toTime, Status = status, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetDocuments", _cert);
            return JsonConvert.DeserializeObject<List<DocumentInfo>>(stringResponse);
        }

        /// <summary>
        /// Delete document from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public bool DeleteDocument(string docId, string groupId)
        {
            DocumentRequest request = new DocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "DeleteDocument", _cert);
            return JsonConvert.DeserializeObject<bool>(stringResponse);
        }

        /// <summary>
        /// Sign document in Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public bool SignDocument(string docId, string groupId, string ssn, string mobile)
        {
            SignDocumentRequest request = new SignDocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username, SSN = ssn, Mobile = mobile };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "SignDocument", _cert);
            return JsonConvert.DeserializeObject<bool>(stringResponse);
        }

        /// <summary>
        /// Get notification from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public ResultStatus GetNotification(string docId, string groupId)
        {
            DocumentRequest request = new DocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetNotification", _cert);
            return JsonConvert.DeserializeObject<ResultStatus>(stringResponse);
        }

        /// <summary>
        /// Remind signers of document in Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public void RemindDocument(string docId, string groupId)
        {
            DocumentRequest request = new DocumentRequest { DocID = docId, GroupID = groupId, password = _password, username = _username };
            string data = JsonConvert.SerializeObject(request);
            CommonHelper.PostData(data, _url + "RemindDocument", _cert);            
        }

        /// <summary>
        /// Get auth token for sign|signsp|signread
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <param name="ssn"></param>
        /// <param name="mobile"></param>
        /// <param name="returnUrl"></param>
        /// <param name="certificate"></param>
        /// <param name="mandate"></param>
        /// <param name="onbehalf"></param>
        /// <returns></returns>
        public string GetToken(string docId, string groupId, string ssn, string mobile, string returnUrl, string certificate, string mandate, string onbehalf)
        {
            TokenRequest request = new TokenRequest 
            {
                DocID = docId, 
                GroupID = groupId, 
                SSN = ssn, 
                Mobile = mobile, 
                ReturnURL = returnUrl, 
                UserCert = certificate, 
                Manadate = mobile, 
                Onbehalf = onbehalf,
                password = _password, 
                username = _username 
            };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetToken", _cert);
            return JsonConvert.DeserializeObject<string>(stringResponse);
        }
            
    }
}
