﻿using Advania.Signet.Cloud2Connector.Models;
using Token = Advania.Signet.Cloud2Connector.Models.Token;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Advania.Signet.Cloud2Connector
{
    public class TokenConnector
    {
        private string _apikey;
        private X509Certificate2 _cert;
        private string _token;
        private string _url;
        private string _username;
        private string _ssn;
        private string _mobile;
        private TokenManager manager;
        private DateTime validTo = DateTime.MinValue;

        /// <summary>
        /// Cloud token constructor
        /// </summary>
        /// <param name="url">Webservice REST url</param>
        /// <param name="username">Signet username</param>
        /// <param name="password">Signet password</param>
        /// <param name="certificate">Signet certificate</param>
        public TokenConnector(string url, string certificate, string username, string password, string issuer)
        {
            _url = url;
            _cert = CommonHelper.GetCertificate(certificate);
            _apikey = CommonHelper.GetApiKey(username, password);
            _username = username;
            manager = new TokenManager(url, issuer, _cert);
        }

        /// <summary>
        /// Set the authentication token
        /// </summary>
        /// <param name="token"></param>
        public void SetToken(string token, string ssn, string mobile)
        {
            _token = token;
            _ssn = ssn;
            _mobile = mobile;
            validTo = TokenManager.GetValidity(token);
        }

        /// <summary>
        /// Get the validity of the token
        /// </summary>
        /// <returns></returns>
        public DateTime GetValidity()
        {
            return validTo;
        }


        /// <summary>
        /// Get account token Signet
        /// </summary>
        /// <param name="SSN"></param>
        /// <param name="Mobile"></param>
        /// <returns></returns>
        public string GetToken(string SSN, string Mobile)
        {
            Token.TokenRequest request = new Token.TokenRequest
            {
                SAML = manager.CreateSaml(_username),
                SSN = SSN,
                Phone = Mobile
            };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetToken", null, false, _apikey);
            string token = JsonConvert.DeserializeObject<string>(stringResponse);
            SetToken(token, SSN, Mobile);
            return _token;
        }

        /// <summary>
        /// Add document(s) to Signet
        /// </summary>
        /// <param name="documents"></param>
        /// <param name="notes"></param>
        /// <param name="metadata"></param>
        /// <param name="reason"></param>
        /// <param name="signers"></param>
        /// <param name="ordered"></param>
        /// <param name="startSigning"></param>
        /// <param name="endSigning"></param>
        /// <returns></returns>
        public AddDocumentResponse AddDocuments(List<Document> documents, string notes, string metadata, string reason, List<DocumentSigner> signers, bool ordered, string groupId, DateTime? startSigning, DateTime? endSigning)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            AddDocumentRequest req = new AddDocumentRequest
            {
                DocumentNotes = notes,
                Signers = signers,
                Documents = documents,
                Metadata = metadata,
                Reason = reason,
                StartSigning = startSigning,
                EndSigning = endSigning,
                Ordered = ordered,
                MininumSigners = 0
            };
            Token.AddRequest request = new Token.AddRequest { GroupID = groupId, request = req, Token = _token };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "AddDocument", _cert, true, _apikey);
            return JsonConvert.DeserializeObject<AddDocumentResponse>(stringResponse);
        }

        /// <summary>
        /// Download document from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public List<Document> DownloadDocument(string docId, string groupId)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.DocumentRequest request = new Token.DocumentRequest { DocID = docId, GroupID = groupId, Token = _token };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "DownloadDocument", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<List<Document>>(stringResponse);
        }

        /// <summary>
        /// Get document info from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public List<DocumentInfo> GetDocumentInfo(string docId, string groupId)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.DocumentRequest request = new Token.DocumentRequest { DocID = docId, GroupID = groupId, Token = _token };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetDocument", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<List<DocumentInfo>>(stringResponse);
        }

        /// <summary>
        /// Get documents from Signet
        /// </summary>        
        /// <returns></returns>
        public List<DocumentInfo> GetDocuments()
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.BaseRequest request = new Token.BaseRequest { Token = _token };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "Documents", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<List<DocumentInfo>>(stringResponse);
        }

        /// <summary>
        /// Delete document from Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <returns></returns>
        public bool DeleteDocument(string docId, string groupId)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.DocumentRequest request = new Token.DocumentRequest { DocID = docId, GroupID = groupId, Token = _token };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "DeleteDocument", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<bool>(stringResponse);
        }
        
        /// <summary>
        /// Reject document in Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <param name="reason"></param>
        /// <param name="lang"></param>
        /// <param name="auth"></param>
        /// <returns></returns>
        public bool RejectDocument(string docId, string groupId, string reason, string lang, bool auth)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.RejectDocumentRequest request = new Token.RejectDocumentRequest
            {
                Auth = auth,
                DocID = docId,
                GroupID = groupId,                
                Lang = lang,
                Reason = reason,
                Token = _token
            };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "RejectDocument", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<bool>(stringResponse);
        }

        /// <summary>
        /// Sign document in Signet
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>
        /// <param name="lang"></param>
        /// <returns></returns>
        public bool SignDocument(string docId, string groupId, string lang)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.SignDocumentRequest request = new Token.SignDocumentRequest
            {
                DocID = docId,
                GroupID = groupId,
                Lang = lang,
                Token = _token
            };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "SignDocument", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<bool>(stringResponse);
        }

        /// <summary>
        /// Get auth token for sign|signsp|signread with card
        /// </summary>
        /// <param name="docId"></param>
        /// <param name="groupId"></param>        
        /// <param name="returnUrl"></param>
        /// <param name="certificate"></param>        
        /// <returns></returns>
        public string GetSignToken(string docId, string groupId, string returnUrl, string certificate)
        {
            if (_token == null)
                throw new Exception("Auhentication token is null");
            if (validTo < DateTime.Now)
                GetToken(_ssn, _mobile);
            Token.SignTokenRequest request = new Token.SignTokenRequest 
            {
                DocID = docId, 
                GroupID = groupId,                 
                ReturnURL = returnUrl, 
                Token = _token,
                UserCert = certificate
            };
            string data = JsonConvert.SerializeObject(request);
            string stringResponse = CommonHelper.PostData(data, _url + "GetSignToken", _cert, false, _apikey);
            return JsonConvert.DeserializeObject<string>(stringResponse);
        }
            
    }
}
